#include "cpp.h"

facility q1 ("queue1");
facility q2 ("queue2");
event done("sim end");
qtable* q1len_qtable;
qtable* q2len_qtable;
table* waiting1_table;
table* waiting2_table;
table* service1_table;
table* service2_table;
stream* st;

double LIFE = 0;
double lambda;
double mu1, mu2;
double p1, p2;
int cust_pass_1 = 0;
int cust_pass_2 = 0;
int cust_leave_1 = 0;
int cust_leave_2 = 0;

//################################################################
// 		functions
//################################################################

extern "C" void sim(int, char **);
extern void read_parm();

int q1len(void) {
	return q1.qlength() + q1.num_busy() ;
}

int q2len(void) {
	return q2.qlength() + q2.num_busy() ;
}

void customer (int i) {
	char msg[256];
	float t1;
	double srv_time;

	create("customer");

	t1 = clock;						// mark arrival time
	q1len_qtable->note_entry();
	sprintf(msg, "Cust %d: join queue 1\n", i);
	trace_msg(msg);

	// in first queue
	q1.reserve();
		sprintf(msg, "Cust %d: arrive at queue 1 head\n", i);
		trace_msg(msg);
		waiting1_table->record( clock-t1 );		// record waiting time
		srv_time = st->exponential( 1.0 / mu1 );	// find service time
		hold( srv_time );
		service1_table->record( srv_time );		// mark service time
		sprintf(msg, "Cust %d : leave queue 1\n", i);
		trace_msg(msg);
	q1.release();

	q1len_qtable->note_exit();
	if ( st->uniform(0, 1) < p1 ) {
		sprintf(msg, "Cust %d : leave system\n", i);
		trace_msg(msg);
		cust_leave_1++;
		return;
	}
	cust_pass_1++;

	t1 = clock;
	q2len_qtable->note_entry();
	sprintf(msg, "Cust %d: join queue 2\n", i);
	trace_msg(msg);

	// in second queue
	q2.reserve();
		sprintf(msg, "Cust %d: arrive at queue 2 head\n", i);
		trace_msg(msg);
		waiting2_table->record( clock-t1 );		// record waiting time
		srv_time = st->exponential( 1.0 / mu2 );	// find service time
		hold( srv_time );
		service2_table->record( srv_time );		// mark service time
		sprintf(msg, "Cust %d : leave queue 2\n", i);
		trace_msg(msg);
	q2.release();

	q2len_qtable->note_exit();
	if ( st->uniform(0, 1) < p2 ) {
		sprintf(msg, "Cust %d : leave system\n", i);
		trace_msg(msg);
		cust_leave_2++;
		return;
	}
	cust_pass_2++;

	customer(i);		// loop back
}


// generate customers
void gen_cust(void) {
	int i = 1;
	create("gen_cust");

        while (1) {
		hold( st->exponential(1.0 / lambda) );	// arrival time
                customer(i++);				// creates a new customer
		if ( clock >= LIFE ) {
			done.set();			// mark the end
		}
        }
}

void my_report (void) {
	printf("\t\t\tpass\tleave\tleave prob\n");
	double prob1 = (double)(cust_leave_1) / (double)(cust_pass_1 + cust_leave_1);
	printf("queue 1\t\t\t%d\t%d\t%f\n", cust_pass_1, cust_leave_1, prob1);
	double prob2 = (double)(cust_leave_2) / (double)(cust_pass_2 + cust_leave_2);
	printf("queue 2\t\t\t%d\t%d\t%f\n", cust_pass_2, cust_leave_2, prob2);
}

void sim(int argc, char *argv[])
{
	set_model_name("flow back model");
	create("sim");
	read_parm();
//	trace_on();

	// initialize tables and stream
	waiting1_table = new table("waiting time 1");
	waiting1_table->confidence();
	waiting1_table->add_histogram(9, 0.5, 5.0);
	waiting2_table = new table("waiting time 2");
	waiting2_table->confidence();
	waiting2_table->add_histogram(9, 0.5, 5.0);
	service1_table = new table("service time 1");
	service1_table->add_histogram(9, 0.5, 5.0);
	service2_table = new table("service time 2");
	service2_table->add_histogram(9, 0.5, 5.0);
	q1len_qtable = new qtable("q1len");
	q2len_qtable = new qtable("q2len");
	st = new stream();

	gen_cust();			// call customer-generating process
	done.wait();			// wait for the event done
	report();			// model report
	mdlstat();			// model statistics
	my_report();
}
