#include "cpp.h"
#define N 10

facility_set q ("queue", N);
event_set service_begin("service begin", N);	//server2job
event done("sim end");
qtable* qlen_qtable[N];
table* waiting_table[N];
table* service_time_table;
stream* st[N];

int server_room = 0;			// 0 means sleep; else cust ID
double LIFE = 0;
int nq = N;				// total num of queues/classes
double lambda[N], mu[N];

//################################################################
// 		functions
//################################################################

extern "C" void sim(int, char **);
extern int policy();
extern void read_parm();

int qlen(int i) {
	return q[i].qlength() + q[i].num_busy() ;
}

int total_qlen() {
	int i;
	int sum = 0;
	for (i=0; i<nq; i++) {
		sum += qlen(i);
	}
	return sum;
}

// class i customer
void customer (int i) {
	char msg[256];
	float t1;
	double srv_time;

	create("customer");
	t1 = clock;
	qlen_qtable[i]->note_entry();

	sprintf(msg, "Cust %d : join queue %d\n", i, i);
	trace_msg(msg);

	q[i].reserve();
		sprintf(msg, "Cust %d : arrive at queue head", i);
		trace_msg(msg);
		if (server_room != 0)
			service_begin[i].wait();

		server_room = identity();
		sprintf(msg, "Cust %d : start service\n", i);
		trace_msg(msg);
		// record waiting time
		waiting_table[i]->record(clock-t1);
	q[i].release();

	srv_time = st[i]->exponential( 1.0 / mu[i] );
	service_time_table->record(srv_time);
	hold(srv_time);

	sprintf(msg, "Cust %d : end service\n", i);
	trace_msg(msg);

	server_room = 0;
	qlen_qtable[i]->note_exit();

	if (total_qlen() > 0) {
		i = policy();
		service_begin[i].set();
	}

	if ( clock >= LIFE )
		done.set();
}

// generate class i customer
void gen_cust(int i) {
	create("gen_cust");
	while (1) {
		hold( st[i]->exponential(1.0 / lambda[i]) );
		customer(i);		// generate next customer
        }
}

void sim(int argc, char *argv[])
{
	int i;

	set_model_name("priority classes");
	create("sim");
	read_parm();

	for (i=0; i<N; i++) {
		waiting_table[i] = new table("waiting time");
		waiting_table[i]->confidence();
		waiting_table[i]->add_histogram(9, 0.5, 5.0);
		qlen_qtable[i] = new qtable("qlen");
		st[i] = new stream(i);
	}

	service_time_table = new table("service time for all class");

	for (i=0; i<nq; i++) {
		gen_cust(i);
	}

	done.wait();
	report();			// model report
	mdlstat();			// model statistics
}
